/******************************************************************************
 * Copyright (C) 2009. Filip Maric
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 ******************************************************************************/
#include <iostream>
#include <fstream>
#include <cstdlib>
#include <vector>
using namespace std;

#include "urm.hpp"
#include "urm-instruction.hpp"


int main(int argc, char* argv[]) {
  if (argc <= 1) {
    cerr << "Usage: " << argv[0] << " <filename>" << endl;
    exit(EXIT_FAILURE);
  }

  ifstream in(argv[1]);
  if (in.fail()) {
    cerr << "Could not open " << argv[1] << endl;
    exit(EXIT_FAILURE);
  }

  URM urm;

  try {
    urm.parse(in);

    cout << "Machine: " << urm.numInstructions() << " instructions" << endl;
    urm.printInstructions(cout);

    cout << "Starting simulation ..." << endl;
    urm.execute();

    cout << "Machine halts in state: " << endl;
    urm.printState(cout);

    in.close();

  } catch (string str) {
    cerr << str << endl;
    exit(EXIT_FAILURE);
  }

  return 0;
}
