/******************************************************************************
 * Copyright (C) 2009. Filip Maric
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 ******************************************************************************/
#include "urm.hpp"

#include <cstdlib>

#include "urm-instruction.hpp"

URM::~URM() {
  for (vector<URMInstruction*>::iterator i = _instructions.begin();
       i != _instructions.end();
       i++)
    delete *i;
  delete _trace;
}

void URM::execute() {
  while(true) {
    URMInstruction* i = get_instruction(_currentInstruction);
    if (i == 0)
      break;
    i->executeAndTrace(*this);
  }
}

void URM::printInstructions(ostream& ostr) const {
  int no = 1;
  for (vector<URMInstruction*>::const_iterator i = _instructions.begin();
       i != _instructions.end();
       i++) {
    ostr << no++ << " : ";
    (*i)->print(ostr);
  }
}

void URM::parse(istream& in) {
  string line;
  while (getline(in, line)) {
    if (line != "") {
      if (line[0] == ':')
	addLabel(line.substr(1));
      else
	addInstruction(URMInstruction::parse(line));
    }
  }
}

